@@ -6,7 +6,7 @@
         my $self = shift;
         my ( $controller, $c ) = @_;
         $self->prepare_soap_helper($controller,$c);
-        $self->NEXT::execute(@_);
+        $self->next::method(@_);
     }
 };
 
@@ -35,7 +35,7 @@ There is not much to be done here.
 
 =head1 AUTHORS
 
-Daniel Ruoso <daniel.ruoso@verticalone.pt>
+Daniel Ruoso <daniel@ruoso.com>
 
 =head1 BUG REPORTS
 
