@@ -18,7 +18,7 @@
           my @children = grep { UNIVERSAL::isa( $_, 'XML::LibXML::Element') } $body->getChildNodes();
           if (scalar @children != 1) {
               $c->stash->{soap}->fault
-                ({ code => 'Client',
+                ({ code => 'SOAP-ENV:Client',
                    reason => 'Bad Body', detail =>
                    'RPC messages should contain only one element inside body'})
             } else {
@@ -32,7 +32,8 @@
                 eval {
                     if ($controller->wsdlobj) {
                         my $decoder = $controller->decoders->{$operation};
-                        my ($args) = $decoder->($rpc_element);
+                        my ($args) = $decoder->($rpc_element)
+                          if UNIVERSAL::isa($decoder,'CODE');
                         $c->stash->{soap}->arguments($args);
                     } else {
                         my $arguments = $rpc_element->getChildNodes();
@@ -41,7 +42,7 @@
                 };
                 if ($@) {
                     $c->stash->{soap}->fault
-                      ({ code => 'Client',
+                      ({ code => 'SOAP-ENV:Client',
                          reason => 'Bad Body', detail =>
                          'Malformed parts on the message body: '.$@});
                 } else {
@@ -50,7 +51,7 @@
                     if (!$action ||
                         !grep { /RPC(Encoded|Literal)/ } @{$action->attributes->{ActionClass}}) {
                         $c->stash->{soap}->fault
-                          ({ code => 'Client',
+                          ({ code => 'SOAP-ENV:Client',
                              reason => 'Bad Operation', detail =>
                              'Invalid Operation'});
                     } else {
@@ -93,7 +94,7 @@ made.
 
 =head1 AUTHORS
 
-Daniel Ruoso <daniel.ruoso@verticalone.pt>
+Daniel Ruoso <daniel@ruoso.com>
 
 =head1 BUG REPORTS
 
